-----------------------------------------------------------------------------
-- |
-- License     :  BSD-3-Clause
-- Maintainer  :  Oleg Grenrus <oleg.grenrus@iki.fi>
--
-- This module re-exports the @GitHub.Data.Enterprise.@ submodules.
module GitHub.Data.Enterprise (
    -- * Module re-exports
    module GitHub.Data.Enterprise.Organizations,
    ) where

import GitHub.Data.Enterprise.Organizations
